// This file is part of the FlexVer™ reference implementation
//
// © 2017 Raptor Engineering, LLC
// All Rights Reserved
//
// WARNING
// This material may be covered by patent(s) and/or patent(s) pending
//
// Any use of this file without an active FlexVer™ license from
// Raptor Engineering, LLC is strictly prohibited.  A single
// exception to this is made for the sole purpose of third-party
// security auditing of FlexVer™ and associated technologies.
//
// For the security audit exception to apply, the primary purpose
// of the use must be to audit the security of the FlexVer™ technology,
// and Raptor Engineering, LLC must be notified in writing of
// any security flaw(s) or potential security flaw(s) found during
// the audit no less than four (4) weeks in advance of public
// disclosure of said flaw(s).  Furthermore, the number of compiled
// FlexVer™ binaries must be kept to the minimum necessary to conduct
// the security audit, and all compiled FlexVer™ binaries must be
// destroyed once the audit has been completed.  These restrictions
// do not apply to organizations or individuals possessing an active
// FlexVer™ license from Raptor Engineering, LLC.
//
// For sales and licensing information, please visit
// https://www.raptorengineering.com
//
// PUBLIC VERSION FOR GENERAL REDISTRIBUTION
// Licensed for use with Raptor Data Solutions, LLC's IntegriCloud™ service

#ifndef LIBFLEXVER_H
#define LIBFLEXVER_H

#define FLEXVER_STATUS_INITIALIZED		0x00000001
#define FLEXVER_STATUS_GENERATING_PUBLIC_KEY	0x00000002
#define FLEXVER_STATUS_GENERATING_SIGNATURE	0x00000004
#define FLEXVER_STATUS_GENERATING_HASH		0x00000008
#define FLEXVER_STATUS_CHECKING_FIRMWARE	0x00000010
#define FLEXVER_STATUS_SECURE			0x00000020
#define FLEXVER_STATUS_HOST_ONLINE		0x00000040
#define FLEXVER_STATUS_HOST_SECURE		0x00000080
#define FLEXVER_STATUS_HASH_OBSOLETE		0x00000100
#define FLEXVER_STATUS_HOST_FW_UPDATE		0x00000200
#define FLEXVER_STATUS_HOST_RESET_ACTIVE	0x00000400
#define FLEXVER_STATUS_HOST_RESET_MISSED	0x00000800
#define FLEXVER_STATUS_HOST_POWER_ACTIVE	0x00001000
#define FLEXVER_STATUS_HOST_STANDBY_POWER_ACT	0x00002000
#define FLEXVER_STATUS_TRANSFERRING_FIRMWARE	0x00004000
#define FLEXVER_STATUS_SEC_HASH_STORAGE_LOCKED	0x20000000
#define FLEXVER_STATUS_KEY_STORAGE_LOCKED	0x40000000
#define FLEXVER_STATUS_OPERATION_QUEUED		0x80000000

#define FLEXVER_CONTROL_UPDATE_HASH		0x1
#define FLEXVER_CONTROL_UPDATE_SIGNATURE	0x2
#define FLEXVER_CONTROL_ALLOW_HOST_FW_UPDATE	0x4
#define FLEXVER_CONTROL_HOST_SIGNATURE_ERROR	0x8

typedef enum {
	FLEXVER_LOGLEVEL_INFO = 3,
	FLEXVER_LOGLEVEL_WARN = 2,
	FLEXVER_LOGLEVEL_ERR = 1,
	FLEXVER_LOGLEVEL_NONE = 0,
} flexver_log_level;

typedef void (*flexver_opwait_status_callback)(void);

void set_flexver_loglevel(flexver_log_level log_level);
uint8_t get_secure_state_from_flexver_status_field(uint32_t status, uint8_t signature_valid, uint8_t assert_security_violation);
char* decode_flexver_status_field(uint32_t status, uint8_t signature_valid, uint8_t assert_security_violation, uint8_t format_dual_line, char** second_line);
int extract_flexver_ssh_public_key_components(uint8_t* flexver_ssh_public_key, uint8_t* exponent, uint8_t* modulus, uint32_t* exponent_length, uint32_t* modulus_length);
int verify_flexver_core_integrity_measurements(uint32_t flexver_status, uint16_t flexver_internal_boot_counter, uint8_t* flexver_public_key, uint8_t* flexver_signature, uint8_t* flexver_primary_firmware_hash, uint8_t* flexver_secondary_firmware_hash, uint8_t* flexver_signature_nonce, uint8_t* flexver_ssh_public_key, uint8_t* signature_valid, uint32_t* flexver_status_field, uint16_t* flexver_internal_boot_count);
#endif // LIBFLEXVER_H
